@@ -732,24 +732,28 @@ get_value(grn_hash *hash, entry_str *n)
   }
 }
 
-inline static uint32_t
-put_key_(grn_ctx *ctx, grn_hash *hash, const char *key, int len)
+inline static void
+put_key_(grn_ctx *ctx, grn_hash *hash, entry_str *n, const char *key, int len)
 {
   uint32_t res, ts;
-  if (len >= GRN_HASH_SEGMENT_SIZE) { return 0; /* error */ }
-  res = hash->header->curr_key;
-  ts = (res + len) >> W_OF_KEY_IN_A_SEGMENT;
-  if (res >> W_OF_KEY_IN_A_SEGMENT != ts) {
-    res = hash->header->curr_key = ts << W_OF_KEY_IN_A_SEGMENT;
+  if (n->size) {
+    res = n->str;
+  } else {
+    if (len >= GRN_HASH_SEGMENT_SIZE) { return; /* error */ }
+    res = hash->header->curr_key;
+    ts = (res + len) >> W_OF_KEY_IN_A_SEGMENT;
+    if (res >> W_OF_KEY_IN_A_SEGMENT != ts) {
+      res = hash->header->curr_key = ts << W_OF_KEY_IN_A_SEGMENT;
+    }
+    hash->header->curr_key += len;
+    n->str = res;
   }
   {
     uint8_t *dest;
     KEY_AT(hash, res, dest);
-    if (!dest) { return 0; }
+    if (!dest) { return; }
     memcpy(dest, key, len);
   }
-  hash->header->curr_key += len;
-  return res;
 }
 
 inline static grn_rc
@@ -763,7 +767,7 @@ put_key(grn_ctx *ctx, grn_hash *hash, entry_str *n, uint32_t h, const char *key,
         memcpy(&n->str, key, len);
       } else {
         n->flag = 0;
-        n->str = put_key_(ctx, hash, key, len);
+        put_key_(ctx, hash, n, key, len);
       }
     } else {
       if (len <= sizeof(char *)) {
@@ -771,7 +775,7 @@ put_key(grn_ctx *ctx, grn_hash *hash, entry_str *n, uint32_t h, const char *key,
         memcpy(&((entry_astr *)n)->str, key, len);
       } else {
         grn_ctx *ctx = hash->ctx;
-        if (!(((entry_astr *)n)->str = GRN_CTX_ALLOC(ctx, len, 0))) {
+        if (!(((entry_astr *)n)->str = GRN_CTX_ALLOC(ctx, len))) {
           return GRN_NO_MEMORY_AVAILABLE;
         }
         memcpy(((entry_astr *)n)->str, key, len);
@@ -879,7 +883,7 @@ tiny_hash_init(grn_hash *ah, grn_ctx *ctx, const char *path, uint32_t key_size,
 {
   uint32_t entry_size;
   if (path) { return GRN_INVALID_ARGUMENT; }
-  if (!(ah->index = GRN_CTX_ALLOC(ctx, INITIAL_INDEX_SIZE * sizeof(grn_id), 0))) {
+  if (!(ah->index = GRN_CTX_ALLOC(ctx, INITIAL_INDEX_SIZE * sizeof(grn_id)))) {
     return GRN_NO_MEMORY_AVAILABLE;
   }
   if (flags & GRN_OBJ_KEY_VAR_SIZE) {
@@ -1086,7 +1090,7 @@ grn_hash_reset(grn_ctx *ctx, grn_hash *hash, uint32_t ne)
     }
   } else {
     GRN_ASSERT(ctx == hash->ctx);
-    if (!(index = GRN_CTX_ALLOC(ctx, n * sizeof(grn_id), 0))) {
+    if (!(index = GRN_CTX_ALLOC(ctx, n * sizeof(grn_id)))) {
       return GRN_NO_MEMORY_AVAILABLE;
     }
     sp = hash->index;
@@ -1147,7 +1151,12 @@ entry_new(grn_ctx *ctx, grn_hash *hash, uint32_t size)
       ENTRY_AT_(hash, e, ee, GRN_TABLE_ADD);
       if (!ee) { return GRN_ID_NIL; }
       hh->garbages[size] = ee->key;
-      memset(ee, 0, hh->entry_size);
+      if (hash->obj.header.flags & GRN_OBJ_KEY_VAR_SIZE) {
+        /* keep ee->size && ee->str */
+        memset(((entry_str *)ee)->dummy, 0, hh->value_size);
+      } else {
+        memset(ee, 0, hh->entry_size);
+      }
     } else {
       e = ++hh->curr_rec;
     }
