@@ -1735,9 +1735,9 @@ grn_str_tok(const char *str, size_t str_len, char delim, const char **tokbuf, in
   return tok - tokbuf;
 }
 
-inline static void
+inline static int
 op_getopt_flag(int *flags, const grn_str_getopt_opt *o,
-               int argc, char * const argv[], int *i, const char *optvalue)
+               int argc, char * const argv[], int i, const char *optvalue)
 {
   switch (o->op) {
     case getopt_op_none:
@@ -1752,19 +1752,18 @@ op_getopt_flag(int *flags, const grn_str_getopt_opt *o,
       *flags = o->flag;
       break;
     default:
-      return;
+      return i;
   }
   if (o->arg) {
     if (optvalue) {
       *o->arg = (char *)optvalue;
+    } else if (++i < argc) {
+      *o->arg = argv[i];
     } else {
-      if (++(*i) < argc) {
-        *o->arg = argv[*i];
-      } else {
-        /* TODO: error */
-      }
+      return -1;
     }
   }
+  return i;
 }
 
 int
@@ -1787,8 +1786,12 @@ grn_str_getopt(int argc, char * const argv[], const grn_str_getopt_opt *opts,
         for (o = opts; o->opt != '\0' || o->longopt != NULL; o++) {
           if (o->longopt && strlen(o->longopt) == len &&
               !memcmp(v, o->longopt, len)) {
-            op_getopt_flag(flags, o, argc, argv, &i,
-                           (*eq == '\0' ? NULL : eq + 1));
+            i = op_getopt_flag(flags, o, argc, argv, i,
+                               (*eq == '\0' ? NULL : eq + 1));
+            if (i < 0) {
+              fprintf(stderr, "%s: option '--%s' needs argument.\n", argv[0], o->longopt);
+              return -1;
+            }
             found = 1;
             break;
           }
@@ -1800,7 +1803,11 @@ grn_str_getopt(int argc, char * const argv[], const grn_str_getopt_opt *opts,
           found = 0;
           for (o = opts; o->opt != '\0' || o->longopt != NULL; o++) {
             if (o->opt && *p == o->opt) {
-              op_getopt_flag(flags, o, argc, argv, &i, NULL);
+              i = op_getopt_flag(flags, o, argc, argv, i, NULL);
+              if (i < 0) {
+                fprintf(stderr, "%s: option '-%c' needs argument.\n", argv[0], *p);
+                return -1;
+              }
               found = 1;
               break;
             }
@@ -1814,7 +1821,7 @@ grn_str_getopt(int argc, char * const argv[], const grn_str_getopt_opt *opts,
   }
   return i;
 exit:
-  fprintf(stderr, "cannot recognize option '%s'.\n", argv[i]);
+  fprintf(stderr, "%s: cannot recognize option '%s'.\n", argv[0], argv[i]);
   return -1;
 }
 
@@ -2411,9 +2418,9 @@ grn_text_atoj(grn_ctx *ctx, grn_obj *bulk, grn_obj *obj, grn_id id)
 {
   int vs;
   grn_obj buf;
-  GRN_TEXT_INIT(&buf, 0);
   if (obj->header.type == GRN_ACCESSOR) {
     grn_accessor *a = (grn_accessor *)obj;
+    GRN_TEXT_INIT(&buf, 0);
     for (;;) {
       GRN_BULK_REWIND(&buf);
       switch (a->action) {
@@ -2445,11 +2452,11 @@ grn_text_atoj(grn_ctx *ctx, grn_obj *bulk, grn_obj *obj, grn_id id)
         buf.header.domain = GRN_DB_INT32;
         break;
       case GRN_ACCESSOR_GET_COLUMN_VALUE :
-        /* todo : support vector */
-        grn_obj_get_value(ctx, a->obj, id, &buf);
         if ((a->obj->header.flags & GRN_OBJ_COLUMN_TYPE_MASK) == GRN_OBJ_COLUMN_VECTOR) {
-          grn_id *idp = (grn_id *)GRN_BULK_HEAD(&buf);
           if (a->next) {
+            grn_id *idp;
+            grn_obj_get_value(ctx, a->obj, id, &buf);
+            idp = (grn_id *)GRN_BULK_HEAD(&buf);
             GRN_TEXT_PUTC(ctx, bulk, '[');
             for (vs = GRN_BULK_VSIZE(&buf) / sizeof(grn_id); vs--; idp++) {
               grn_text_atoj(ctx, bulk, (grn_obj *)a->next, *idp);
@@ -2457,19 +2464,11 @@ grn_text_atoj(grn_ctx *ctx, grn_obj *bulk, grn_obj *obj, grn_id id)
             }
             GRN_TEXT_PUTC(ctx, bulk, ']');
           } else {
-            grn_obj b;
-            GRN_RECORD_INIT(&b, 0, DB_OBJ(a->obj)->range);
-            GRN_TEXT_PUTC(ctx, bulk, '[');
-            /* todo: support other fixe sized data types */
-            for (vs = GRN_BULK_VSIZE(&buf) / sizeof(grn_id); vs--; idp++) {
-              GRN_RECORD_SET(ctx, &b, *idp);
-              grn_text_otoj(ctx, bulk, &b, NULL);
-              if (vs) { GRN_TEXT_PUTC(ctx, bulk, ','); }
-            }
-            GRN_TEXT_PUTC(ctx, bulk, ']');
-            GRN_OBJ_FIN(ctx, &b);
+            grn_text_atoj(ctx, bulk, a->obj, id);
           }
           goto exit;
+        } else {
+          grn_obj_get_value(ctx, a->obj, id, &buf);
         }
         break;
       case GRN_ACCESSOR_GET_DB_OBJ :
@@ -2490,6 +2489,29 @@ grn_text_atoj(grn_ctx *ctx, grn_obj *bulk, grn_obj *obj, grn_id id)
       }
     }
   } else {
+    switch (obj->header.type) {
+    case GRN_COLUMN_FIX_SIZE :
+      GRN_VALUE_FIX_SIZE_INIT(&buf, 0, DB_OBJ(obj)->range);
+      break;
+    case GRN_COLUMN_VAR_SIZE :
+      if ((obj->header.flags & GRN_OBJ_COLUMN_TYPE_MASK) == GRN_OBJ_COLUMN_VECTOR) {
+        grn_obj *range = grn_ctx_at(ctx, DB_OBJ(obj)->range);
+        if (range->header.flags & GRN_OBJ_KEY_VAR_SIZE) {
+          GRN_VALUE_VAR_SIZE_INIT(&buf, GRN_OBJ_VECTOR, DB_OBJ(obj)->range);
+        } else {
+          GRN_VALUE_FIX_SIZE_INIT(&buf, GRN_OBJ_VECTOR, DB_OBJ(obj)->range);
+        }
+      } else {
+        GRN_VALUE_VAR_SIZE_INIT(&buf, 0, DB_OBJ(obj)->range);
+      }
+      break;
+    case GRN_COLUMN_INDEX :
+      GRN_UINT32_INIT(&buf, 0);
+      break;
+    default:
+      GRN_TEXT_INIT(&buf, 0);
+      break;
+    }
     grn_obj_get_value(ctx, obj, id, &buf);
   }
   grn_text_otoj(ctx, bulk, &buf, NULL);
@@ -2700,6 +2722,12 @@ grn_text_otoj(grn_ctx *ctx, grn_obj *bulk, grn_obj *obj, grn_obj_format *format)
             grn_text_otoj(ctx, bulk, &buf, NULL);
           }
           GRN_TEXT_PUTC(ctx, bulk, ']');
+          v++;
+          if (v < ve) {
+            GRN_TEXT_PUTC(ctx, bulk, ',');
+          } else {
+            break;
+          }
         }
       }
       GRN_TEXT_PUTC(ctx, bulk, ']');
@@ -2784,6 +2812,24 @@ grn_text_otoj(grn_ctx *ctx, grn_obj *bulk, grn_obj *obj, grn_obj_format *format)
       GRN_TEXT_PUTC(ctx, bulk, ']');
     }
     break;
+  case GRN_PVECTOR :
+    if (format) {
+      ERR(GRN_FUNCTION_NOT_IMPLEMENTED,
+          "cannot print GRN_PVECTOR using grn_obj_format");
+    } else {
+      unsigned int i, n;
+      GRN_TEXT_PUTC(ctx, bulk, '[');
+      n = GRN_BULK_VSIZE(obj) / sizeof(grn_obj *);
+      for (i = 0; i < n; i++) {
+        grn_obj *value;
+
+        if (i) { GRN_TEXT_PUTC(ctx, bulk, ','); }
+        value = GRN_PTR_VALUE_AT(obj, i);
+        grn_text_otoj(ctx, bulk, value, NULL);
+      }
+      GRN_TEXT_PUTC(ctx, bulk, ']');
+    }
+    break;
   case GRN_TABLE_HASH_KEY :
   case GRN_TABLE_PAT_KEY :
   case GRN_TABLE_NO_KEY :
@@ -3116,7 +3162,7 @@ grn_str_url_path_normalize(grn_ctx *ctx, const char *path, size_t path_len,
       memcpy(b, p, (pc - p));
       b += pc - p;
       p = pc;
-      if (*pc == '/' && be > b) {
+      if (p < pe && *pc == '/' && be > b) {
         *b++ = PATH_SEPARATOR[0];
         p++;
       }
