@@ -14,10 +14,6 @@
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
-#ifndef NO_MECAB
-#include <mecab.h>
-#endif /* NO_MECAB */
-
 #ifndef GRN_TOKEN_H
 #define GRN_TOKEN_H
 
@@ -80,6 +76,7 @@ grn_token *grn_token_open(grn_ctx *ctx, grn_obj *table, const char *str,
 grn_id grn_token_next(grn_ctx *ctx, grn_token *ng);
 grn_rc grn_token_close(grn_ctx *ctx, grn_token *ng);
 
+grn_rc grn_db_init_mecab_tokenizer(grn_ctx *ctx);
 grn_rc grn_db_init_builtin_tokenizers(grn_ctx *ctx);
 
 #ifdef __cplusplus
