@@ -39,8 +39,10 @@ void test_text_to_geo_point(void);
 void test_text_to_geo_point_comma(void);
 void test_text_to_geo_point_invalid(void);
 void test_text_to_geo_point_in_degree(void);
+void test_text_to_geo_point_in_degree_comma(void);
 void test_text_to_geo_point_in_degree_invalid(void);
 void test_text_to_geo_point_mixed(void);
+void test_text_to_geo_point_mixed_comma(void);
 void test_text_to_geo_point_mixed_invalid(void);
 
 void data_text_error(void);
@@ -289,8 +291,20 @@ test_text_to_geo_point_in_degree(void)
   grn_obj_reinit(&context, &dest, GRN_DB_WGS84_GEO_POINT, 0);
   cast_text("35.6954581363924x139.564207350021");
   GRN_GEO_POINT_VALUE(&dest, takane_latitude, takane_longitude);
-  cut_assert_equal_int(130194581, takane_latitude);
-  cut_assert_equal_int(503802073, takane_longitude);
+  cut_assert_equal_int(128503649, takane_latitude);
+  cut_assert_equal_int(502431146, takane_longitude);
+}
+
+void
+test_text_to_geo_point_in_degree_comma(void)
+{
+  gint takane_latitude, takane_longitude;
+
+  grn_obj_reinit(&context, &dest, GRN_DB_WGS84_GEO_POINT, 0);
+  cast_text("35.6954581363924,139.564207350021");
+  GRN_GEO_POINT_VALUE(&dest, takane_latitude, takane_longitude);
+  cut_assert_equal_int(128503649, takane_latitude);
+  cut_assert_equal_int(502431146, takane_longitude);
 }
 
 void
@@ -310,7 +324,19 @@ test_text_to_geo_point_mixed(void)
   grn_obj_reinit(&context, &dest, GRN_DB_WGS84_GEO_POINT, 0);
   cast_text("35.6954581363924x503802073");
   GRN_GEO_POINT_VALUE(&dest, takane_latitude, takane_longitude);
-  cut_assert_equal_int(130194581, takane_latitude);
+  cut_assert_equal_int(128503649, takane_latitude);
+  cut_assert_equal_int(503802073, takane_longitude);
+}
+
+void
+test_text_to_geo_point_mixed_comma(void)
+{
+  gint takane_latitude, takane_longitude;
+
+  grn_obj_reinit(&context, &dest, GRN_DB_WGS84_GEO_POINT, 0);
+  cast_text("35.6954581363924,503802073");
+  GRN_GEO_POINT_VALUE(&dest, takane_latitude, takane_longitude);
+  cut_assert_equal_int(128503649, takane_latitude);
   cut_assert_equal_int(503802073, takane_longitude);
 }
 
