@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2; coding: utf-8 -*- */
 /*
-  Copyright (C) 2008-2009  Kouhei Sutou <kou@cozmixng.org>
+  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -42,6 +42,8 @@ void data_set_value_with_null_value(void);
 void test_set_value_with_null_value(gconstpointer data);
 void data_add_and_delete(void);
 void test_add_and_delete(gconstpointer data);
+void data_truncate(void);
+void test_truncate(gconstpointer data);
 
 static GArray *ids;
 
@@ -64,11 +66,18 @@ cut_setup(void)
   ids = NULL;
 }
 
-void
-cut_teardown(void)
+static void
+ids_free(void)
 {
   if (ids)
     g_array_free(ids, TRUE);
+  ids = NULL;
+}
+
+void
+cut_teardown(void)
+{
+  ids_free();
   teardown_hash_common();
 }
 
@@ -208,6 +217,22 @@ test_data_free(grn_test_data *test_data)
   g_free(test_data);
 }
 
+static void
+test_data_add_n_data(guint n, grn_test_data *test_data)
+{
+  guint i;
+
+  ids_free();
+  ids = g_array_new(TRUE, TRUE, sizeof(grn_id));
+  for (i = 0; i < n; i++) {
+    if (grn_test_hash_factory_get_flags(factory) & GRN_OBJ_KEY_VAR_SIZE)
+      key_size = strlen(test_data->key);
+    cut_assert_lookup_add(test_data->key);
+    test_data->increment((grn_test_data *)test_data);
+    g_array_append_val(ids, id);
+  }
+}
+
 static gpointer
 uint32_key_new(uint32_t key)
 {
@@ -540,23 +565,18 @@ data_add_and_delete(void)
 void
 test_add_and_delete(gconstpointer data)
 {
-  const grn_test_data *test_data = data;
+  grn_test_data *test_data;
   guint i;
-  const guint n_operations = 7500;
+  const guint n_operations = 750;
+
+  test_data = (grn_test_data *)data;
 
   if (test_data->set_parameters)
     test_data->set_parameters();
 
   cut_assert_create_hash();
 
-  ids = g_array_new(TRUE, TRUE, sizeof(grn_id));
-  for (i = 0; i < n_operations; i++) {
-    if (grn_test_hash_factory_get_flags(factory) & GRN_OBJ_KEY_VAR_SIZE)
-      key_size = strlen(test_data->key);
-    cut_assert_lookup_add(test_data->key);
-    test_data->increment((grn_test_data *)test_data);
-    g_array_append_val(ids, id);
-  }
+  test_data_add_n_data(n_operations, test_data);
 
   cut_assert_equal_int(n_operations, GRN_HASH_SIZE(hash));
   for (i = 0; i < ids->len; i++) {
@@ -568,3 +588,28 @@ test_add_and_delete(gconstpointer data)
   }
   cut_assert_equal_int(0, GRN_HASH_SIZE(hash));
 }
+
+void
+data_truncate(void)
+{
+  add_variable_key_size_test_data();
+}
+
+void
+test_truncate(gconstpointer data)
+{
+  grn_test_data *test_data;
+  guint n_data = 100;
+
+  test_data = (grn_test_data *)data;
+  if (test_data->set_parameters)
+    test_data->set_parameters();
+
+  cut_assert_create_hash();
+
+  cut_assert_equal_uint(0, GRN_HASH_SIZE(hash));
+  test_data_add_n_data(n_data, test_data);
+  cut_assert_equal_uint(n_data, GRN_HASH_SIZE(hash));
+  grn_test_assert(grn_hash_truncate(context, hash));
+  cut_assert_equal_uint(0, GRN_HASH_SIZE(hash));
+}
