@@ -103,15 +103,16 @@ test_in_circle(void)
   gint distance = 3 * 1000;
 
   cut_assert_equal_string(
-    "[[[6],"
+    "[[[7],"
     "[[\"name\",\"ShortText\"],[\"_score\",\"Int32\"],"
     "[\"location\",\"WGS84GeoPoint\"]],"
-    "[\"銀座 かずや\",795,\"130055008x504968095\"],"
-    "[\"たいやき神田達磨 八重洲店\",1079,\"130094061x505025099\"],"
-    "[\"たい焼き鉄次 大丸東京店\",1390,\"130089012x505045070\"],"
-    "[\"築地 さのきや\",1723,\"130019020x505027021\"],"
-    "[\"にしみや 甘味処\",2000,\"130038025x505066028\"],"
-    "[\"しげ田\",2272,\"130046026x505082073\"]"
+    "[\"銀座 かずや\",280,\"128424629x503139222\"],"
+    "[\"たい焼き鉄次 大丸東京店\",810,\"128451283x503166852\"],"
+    "[\"たいやき神田達磨 八重洲店\",970,\"128453260x503174156\"],"
+    "[\"にしみや 甘味処\",1056,\"128418570x503188661\"],"
+    "[\"築地 さのきや\",1186,\"128397312x503174596\"],"
+    "[\"しげ田\",1530,\"128421454x503208983\"],"
+    "[\"柳屋 たい焼き\",2179,\"128467228x503222332\"]"
     "]]",
     send_command(
       cut_take_printf(
@@ -139,9 +140,9 @@ test_in_rectangle_long_longitude(void)
     "[[[3],"
     "[[\"name\",\"ShortText\"],[\"_score\",\"Int32\"],"
      "[\"location\",\"WGS84GeoPoint\"]],"
-    "[\"たいやき神田達磨 八重洲店\",3273,\"130094061x505025099\"],"
-    "[\"銀座 かずや\",3713,\"130055008x504968095\"],"
-    "[\"たい焼き鉄次 大丸東京店\",3732,\"130089012x505045070\"]"
+    "[\"たい焼き鉄次 大丸東京店\",2186,\"128451283x503166852\"],"
+    "[\"たいやき神田達磨 八重洲店\",2296,\"128453260x503174156\"],"
+    "[\"銀座 かずや\",2415,\"128424629x503139222\"]"
     "]]",
     send_command(
       cut_take_printf(
@@ -167,13 +168,14 @@ test_in_rectangle_long_latitude(void)
   gdouble budoukan_longitude = 139.74968;
 
   cut_assert_equal_string(
-    "[[[4],"
+    "[[[5],"
     "[[\"name\",\"ShortText\"],[\"_score\",\"Int32\"],"
      "[\"location\",\"WGS84GeoPoint\"]],"
-    "[\"たいやき神田達磨 八重洲店\",3273,\"130094061x505025099\"],"
-    "[\"銀座 かずや\",3713,\"130055008x504968095\"],"
-    "[\"根津のたいやき\",4754,\"130322053x504985073\"],"
-    "[\"築地 さのきや\",5244,\"130019020x505027021\"]"
+    "[\"たい焼き鉄次 大丸東京店\",2186,\"128451283x503166852\"],"
+    "[\"たいやき神田達磨 八重洲店\",2296,\"128453260x503174156\"],"
+    "[\"銀座 かずや\",2415,\"128424629x503139222\"],"
+    "[\"根津のたいやき\",3210,\"128592911x503145263\"],"
+    "[\"築地 さのきや\",3579,\"128397312x503174596\"]"
     "]]",
     send_command(
       cut_take_printf(
@@ -195,14 +197,15 @@ test_sort(void)
   gint distance = 3 * 1000;
 
   cut_assert_equal_string(
-    "[[[6],"
+    "[[[7],"
     "[[\"name\",\"ShortText\"],[\"_score\",\"Int32\"]],"
-    "[\"銀座 かずや\",795],"
-    "[\"たいやき神田達磨 八重洲店\",1079],"
-    "[\"たい焼き鉄次 大丸東京店\",1390],"
-    "[\"築地 さのきや\",1723],"
-    "[\"にしみや 甘味処\",2000],"
-    "[\"しげ田\",2272]"
+    "[\"銀座 かずや\",280],"
+    "[\"たい焼き鉄次 大丸東京店\",810],"
+    "[\"たいやき神田達磨 八重洲店\",970],"
+    "[\"にしみや 甘味処\",1056],"
+    "[\"築地 さのきや\",1186],"
+    "[\"しげ田\",1530],"
+    "[\"柳屋 たい焼き\",2179]"
     "]]",
     send_command(
       cut_take_printf(
@@ -226,16 +229,16 @@ test_filter_by_tag_and_sort_by_distance_from_tokyo_tocho(void)
   cut_assert_equal_string(
     "[[[31],"
     "[[\"name\",\"ShortText\"],[\"_score\",\"Int32\"]],"
-    "[\"さざれ\",4110],"
-    "[\"広瀬屋\",4149],"
-    "[\"そばたいやき空\",4507],"
-    "[\"たい焼 カタオカ\",5210],"
-    "[\"車\",5600],"
-    "[\"吾妻屋\",6099],"
-    "[\"たいやき工房白家 阿佐ヶ谷店\",7646],"
-    "[\"たいやき本舗 藤家 阿佐ヶ谷店\",7861],"
-    "[\"たいやきひいらぎ\",8463],"
-    "[\"代官山たい焼き黒鯛\",8668]"
+    "[\"さざれ\",2860],"
+    "[\"広瀬屋\",2870],"
+    "[\"そばたいやき空\",3004],"
+    "[\"たい焼 カタオカ\",3347],"
+    "[\"車\",3792],"
+    "[\"吾妻屋\",4166],"
+    "[\"代官山たい焼き黒鯛\",4497],"
+    "[\"たいやきひいらぎ\",4965],"
+    "[\"たいやき工房白家 阿佐ヶ谷店\",5102],"
+    "[\"たいやき本舗 藤家 阿佐ヶ谷店\",5172]"
     "]]",
     send_command(
       cut_take_printf(
@@ -252,14 +255,16 @@ test_in_circle_and_tag(void)
 {
   gdouble tokyo_tocho_latitude = 35.689444;
   gdouble tokyo_tocho_longitude = 139.691667;
-  gint distance = 5 * 1000;
+  gint distance = 4 * 1000;
 
   cut_assert_equal_string(
-    "[[[3],"
+    "[[[5],"
     "[[\"name\",\"ShortText\"],[\"_score\",\"Int32\"]],"
-    "[\"さざれ\",4110],"
-    "[\"広瀬屋\",4150],"
-    "[\"そばたいやき空\",4524]"
+    "[\"さざれ\",2860],"
+    "[\"広瀬屋\",2871],"
+    "[\"そばたいやき空\",3016],"
+    "[\"たい焼 カタオカ\",3353],"
+    "[\"車\",3794]"
     "]]",
     send_command(
       cut_take_printf(
@@ -278,16 +283,16 @@ test_but_white(void)
 {
   gdouble asagaya_latitude = 35.70452;
   gdouble asagaya_longitude = 139.6351;
-  gint distance = 10 * 1000;
+  gint distance = 5 * 1000;
 
   cut_assert_equal_string(
     "[[[5],"
     "[[\"name\",\"ShortText\"],[\"_score\",\"Int32\"]],"
-    "[\"吾妻屋\",2385],"
-    "[\"そばたいやき空\",5592],"
-    "[\"たい焼き / たつみや\",7148],"
-    "[\"さざれ\",7671],"
-    "[\"広瀬屋\",7830]"
+    "[\"吾妻屋\",1198],"
+    "[\"そばたいやき空\",3162],"
+    "[\"たい焼き / たつみや\",4341],"
+    "[\"さざれ\",4637],"
+    "[\"広瀬屋\",4692]"
     "]]",
     send_command(
       cut_take_printf(
@@ -311,27 +316,28 @@ test_drilldown(void)
   gint distance = 10 * 1000;
 
   cut_assert_equal_string(
-    "[[[13],"
+    "[[[23],"
     "[[\"name\",\"ShortText\"],[\"_score\",\"Int32\"]],"
-    "[\"たいやき神田達磨 八重洲店\",1079],"
-    "[\"たい焼き鉄次 大丸東京店\",1390],"
-    "[\"築地 さのきや\",1723],"
-    "[\"にしみや 甘味処\",2000],"
-    "[\"しげ田\",2272],"
-    "[\"柳屋 たい焼き\",3686],"
-    "[\"根津のたいやき\",7812],"
-    "[\"尾長屋 錦糸町店\",8314],"
-    "[\"横浜 くりこ庵 浅草店\",8394],"
-    "[\"たいやきひいらぎ\",9242]],"
-    "[[6],"
+    "[\"たい焼き鉄次 大丸東京店\",810],"
+    "[\"たいやき神田達磨 八重洲店\",970],"
+    "[\"にしみや 甘味処\",1056],"
+    "[\"築地 さのきや\",1186],"
+    "[\"しげ田\",1530],"
+    "[\"柳屋 たい焼き\",2179],"
+    "[\"尾長屋 錦糸町店\",5007],"
+    "[\"根津のたいやき\",5036],"
+    "[\"横浜 くりこ庵 浅草店\",5098],"
+    "[\"たい焼き写楽\",5457]],"
+    "[[7],"
      "[[\"_key\",\"ShortText\"],"
       "[\"name\",\"ShortText\"],"
       "[\"_nsubrecs\",\"Int32\"]],"
      "[\"おでん\",\"\",1],"
-     "[\"たいやき\",\"\",13],"
+     "[\"たいやき\",\"\",23],"
+     "[\"カレー\",\"\",1],"
      "[\"マグロ\",\"\",1],"
      "[\"和菓子\",\"\",1],"
-     "[\"天然\",\"\",3],"
+     "[\"天然\",\"\",4],"
      "[\"白\",\"\",1]],"
     "[[2],"
      "[[\"_key\",\"ShortText\"],"
@@ -372,18 +378,18 @@ test_drilldown_with_broken_reference(void)
   assert_send_commands("delete Areas area0002");
   assert_send_commands("delete Areas area0005");
   cut_assert_equal_string(
-    "[[[13],"
+    "[[[23],"
     "[[\"name\",\"ShortText\"],[\"_score\",\"Int32\"]],"
-    "[\"たいやき神田達磨 八重洲店\",1081],"
-    "[\"たい焼き鉄次 大丸東京店\",1395],"
-    "[\"築地 さのきや\",1725],"
-    "[\"にしみや 甘味処\",2007],"
-    "[\"しげ田\",2281],"
-    "[\"柳屋 たい焼き\",3698],"
-    "[\"根津のたいやき\",7813],"
-    "[\"尾長屋 錦糸町店\",8339],"
-    "[\"横浜 くりこ庵 浅草店\",8409],"
-    "[\"たいやきひいらぎ\",9273]],"
+    "[\"たい焼き鉄次 大丸東京店\",811],"
+    "[\"たいやき神田達磨 八重洲店\",972],"
+    "[\"にしみや 甘味処\",1060],"
+    "[\"築地 さのきや\",1187],"
+    "[\"しげ田\",1537],"
+    "[\"柳屋 たい焼き\",2186],"
+    "[\"尾長屋 錦糸町店\",5024],"
+    "[\"根津のたいやき\",5036],"
+    "[\"横浜 くりこ庵 浅草店\",5106],"
+    "[\"たい焼き写楽\",5464]],"
     "[[1],"
      "[[\"_key\",\"ShortText\"],"
       "[\"name\",\"ShortText\"],"
@@ -413,22 +419,23 @@ test_weight_match(void)
   gint distance = 10 * 1000;
 
   cut_assert_equal_string(
-    "[[[13],"
+    "[[[23],"
     "[[\"name\",\"ShortText\"],[\"_score\",\"Int32\"]],"
-    "[\"たいやき神田達磨 八重洲店\",9922],"
-    "[\"たい焼き鉄次 大丸東京店\",8611],"
-    "[\"築地 さのきや\",8278],"
-    "[\"にしみや 甘味処\",8001],"
-    "[\"しげ田\",7729],"
-    "[\"柳屋 たい焼き\",6315],"
-    "[\"たいやきひいらぎ\",1759],"
-    "[\"尾長屋 錦糸町店\",1687],"
-    "[\"横浜 くりこ庵 浅草店\",1607],"
-    "[\"たい焼き写楽\",651]],"
-    "[[6],"
+    "[\"たいやき神田達磨 八重洲店\",10031],"
+    "[\"たい焼き鉄次 大丸東京店\",9191],"
+    "[\"にしみや 甘味処\",8945],"
+    "[\"築地 さのきや\",8815],"
+    "[\"しげ田\",8471],"
+    "[\"柳屋 たい焼き\",7822],"
+    "[\"たいやきひいらぎ\",5428],"
+    "[\"尾長屋 錦糸町店\",4994],"
+    "[\"横浜 くりこ庵 浅草店\",4903],"
+    "[\"たい焼き写楽\",4544]],"
+    "[[7],"
     "[[\"_key\",\"ShortText\"],[\"_nsubrecs\",\"Int32\"]],"
-    "[\"たいやき\",13],"
-    "[\"天然\",3],"
+    "[\"たいやき\",23],"
+    "[\"天然\",4],"
+    "[\"カレー\",1],"
     "[\"白\",1],"
     "[\"マグロ\",1],"
     "[\"和菓子\",1],"
